/*
 MBC3 RTC LNL - Set RGB LED Colour
 Version: 1.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 26/08/2021
 Last Modified: 26/08/2021
 License: CC-BY-NC-SA
 
 Use this program to set the RGB LED colour on our MBC3 RTC LinkNLoad cart
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <time.h>
#include <signal.h>

#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

#include "setup.h" // See defines, variables, constants, functions here

void sigintHandler(int sig_num) {
	set_mode('0');
	RS232_flushRX(cport_nr);
	gbx_cart_power_down();
	exit(0);
}

int main(int argc, char **argv) {
	signal(SIGINT, sigintHandler);
	
	printf("MBC3 RTC LNL - Set RGB LED Colour by insideGadgets\n");
	printf("##################################################\n");
	
	read_config();
	
	// Open COM port
	if (com_test_port() == 0) {
		printf("Device not connected and couldn't be auto detected. Please make sure the COM port isn't open elsewhere. If you are using Linux/Mac, make sure you use \"sudo\".\n");
		read_one_letter();
		return 1;
	}
	printf("Connected on COM port: %i at ", cport_nr+1);
	
	// Break out of any existing functions on ATmega
	set_mode('0');
	RS232_flushRX(cport_nr);
	
	// Get cartridge mode - Gameboy or Gameboy Advance
	cartridgeMode = request_value(CART_MODE);
	
	// Get PCB version
	gbxcartPcbVersion = request_value(READ_PCB_VERSION);
	xmas_wake_up();
	
	// Increase to 1.7Mbit baud for v1.4 PCB
	#ifdef _WIN32
	if (gbxcartPcbVersion == PCB_1_4 || gbxcartPcbVersion == PCB_1_1) {
		set_mode(USART_1_7M_SPEED);
		RS232_CloseComport(cport_nr);
		delay_ms(200);
		
		if (RS232_OpenComport(cport_nr, 1700000, "8N1") == 0) {
			printf("1.7M Baud\n");
		}
	}
	else {
		printf("1M Baud\n");
	}
	#else
		printf("1M Baud\n");
	#endif
	
	char pcbVersionString[20];
	if (gbxcartPcbVersion == PCB_1_0) {
		strncpy(pcbVersionString, "v1.0", 5);
	}
	else if (gbxcartPcbVersion == PCB_1_1) {
		strncpy(pcbVersionString, "v1.1/v1.2", 10);
	}
	else if (gbxcartPcbVersion == PCB_1_3) {
		strncpy(pcbVersionString, "v1.3", 5);
	}
	else if (gbxcartPcbVersion == PCB_1_4) {
		strncpy(pcbVersionString, "v1.4", 5);
	}
	else if (gbxcartPcbVersion == MINI) {
		strncpy(pcbVersionString, "Mini", 5);
	}
	else if (gbxcartPcbVersion == MINI_1_1) {
		strncpy(pcbVersionString, "Mini v1.1", 10);
	}
	else if (gbxcartPcbVersion == GBXMAS) {
		strncpy(pcbVersionString, "XMAS", 5);
	}
	
	// Get firmware version
	gbxcartFirmwareVersion = request_value(READ_FIRMWARE_VERSION);
	printf("PCB version: %s, Firmware version: %i\n", pcbVersionString, gbxcartFirmwareVersion);
	
	if (gbxcartFirmwareVersion == 0) {
		printf("\nFirmware L1 may be installed. In order to use this applications you will need to downgrade to R30.\n");
		read_one_letter();
		return 1;
	}
	
	set_mode(VOLTAGE_5V);
	
	
	// GBx v1.4 - Power up the cart if not already powered up and flush buffer
	gbx_cart_power_up();
	RS232_flushRX(cport_nr);
	
	if (gbxcartPcbVersion == GBXMAS) {
		set_mode('!');
		delay_ms(50); // Wait for ATmega169 to WDT reset if more than 2 mins idle
	}
	
	// Get cartridge mode - Gameboy or Gameboy Advance
	cartridgeMode = request_value(CART_MODE);
	
	while (1) {
		printf("\nEnter the Red colour brightness: [0-255]\n");
		printf(">");
		char optionString[5];
		fgets(optionString, 5, stdin);
		int redValue = atoi(optionString);
		
		printf("\nEnter the Green colour brightness: [0-255]\n");
		printf(">");
		fgets(optionString, 5, stdin);
		int greenValue = atoi(optionString);
		
		printf("\nEnter the Blue colour brightness: [0-255]\n");
		printf(">");
		fgets(optionString, 5, stdin);
		int blueValue = atoi(optionString);
		
		
		RS232_SendByte(cport_nr, RGB_LED);
		RS232_SendByte(cport_nr, redValue);
		RS232_SendByte(cport_nr, greenValue);
		RS232_SendByte(cport_nr, blueValue);
		RS232_drain(cport_nr);
		delay_ms(5);
		
		printf("\nCompleted\n\n");
	}
	
	return 0;
}
